#!/usr/bin/perl

#Configuartion File having the Site specific Configuration
$CONFIGURATION_FILE = "/opt/gridview/etc/gridview-gridftp-conf.pl";
$WSPUBLISHER_LIB_FILE = "/opt/gridview/sbin/gridview-common-wsclient.pl";

##############################################
#                                            #
#  Please Do not change below this Line      #
#                                            #
##############################################         

#Version of this Publisher. Please Do not change it!!!!!!!
$CONF_VERSION = "1.1.0";

#include the configuration File
do $CONFIGURATION_FILE;

##############################################
#                                            #
#        Setting the Default Values          #
#                                            #
##############################################         

#BASE_LOG_FILE_NAME is used to define the file name of the log file.
#It is BASE bcause in case of Log rotated files the filename is regenerated
#by appending a number to the BASE_LOG_FILE_NAME
$CONF_BASE_LOG_FILE_NAME = "/var/log/globus-gridftp.log";

#MAX_SLEEP_SECOND is used to define the max number of second that 
#program should sleep in case of no records available. 
$CONF_MAX_SLEEP_SECONDS = 60;

#Temporary directory where the publisher should unzip the logrotated files
$CONF_UNZIP_TMP_DIR = "/tmp/";

#File to keep the state of the Publisher. 
#This is to avoid publishing records that are already published.
$CONF_STATEFILE = "/opt/gridview/misc/gridview-gridftp-wsclient.state";

#File to keep the lock for the Publisher.
#This is used to avoid the other instance to start if there is one 
#already running
$CONF_LOCKFILE = "/opt/gridview/misc/gridview-gridftp-wsclient.lock";

#File to keep the log of the Publisher itself
#This is used to log the events of Publisher 
#amount of data written to the file is governed by LOG_LEVEL
$CONF_LOGFILE = "/opt/gridview/log/gridview-grdftp-wsclient";

#WSDL of the Web Service archiver of Gridview.
$CONF_SERVER_WSDL = "http://gvarch.cern.ch:8080/wsarch/services/WebArchiverAdv?wsdl";

#This is the timeout in minutes for detecting the process to be Hanged
$CONF_TIMEOUT = 30;

#This is the Wat Time in minutes that parent process has to wait
#for creating a new child if the child has exited.
$CONF_RETRY_WAIT = 5;

#LOG_LEVEL defines the verbosity of the Publisher
$CONF_LOG_LEVEL = 0;

#DEBUG_LEVEL this value defines where the logs are to written.
# For 0 = only to file, 1 = only to screen, 2 = to screen and File both
#This Option is not available to configuration.
$CONF_DEBUG = 0;

#Setting the TimeZone to UTC to have the same time format
$ENV{TZ} = "UTC";

#TABLE_NAME is the name of table that is used by the archiver 
#to know which table to use
$CONF_TABLE_NAME = "gridftpmonitor";

#TUPLE_ORDER defines the order in which the tuples are to be published.
#Please don't change this without permission of Gridview Team
@CONF_TUPLE_ORDER = ('HOST', 'USER', 'SRC', 'DEST', 'BLOCK', 'STREAMS', 'BUFFER', 'NBYTES', 'STRIPES', 'TXRATE', 'VOLUME', 'FILE', 'CODE', 'SSECONDS', 'SSECONDS_MICRO', 'ESECONDS', 'ESECONDS_MICRO', 'END_DATE', 'END_TIME');

#Including the Library File that holds all the library functions
do $WSPUBLISHER_LIB_FILE;

#Calling the Main Program
&main;

#This function  returns the header of the WS publisher
sub getHeader {
        return "";
}

#Parses the Line and breaks them in fields to be used to make string 
#to be published
#Specific to the GridFTP log file format.
sub parseLine
{
        my $inline = shift;
	my %record;
        #This regular Expression is for Parsing Each Record of GridFTP log

        if( $inline !~ /\s*DATE=(.*)\s+HOST=(.*)\s+PROG=(.*)\s+NL\.EVNT=(.*)\s+START=(\S*)(?:\s+USER=(.*))?\s+FILE=(.*)\s+BUFFER=(.*)\s+BLOCK=(.*)\s+NBYTES=(.*)\s+VOLUME=(.*)\s+STREAMS=(.*)\s+STRIPES=(.*)\s+DEST=\d*\[(.*)\]\s+TYPE=(.*)\s+CODE=(.*).*/)
        {
		&logMessage (5, "parseLine: Error Input Line not matches $inline"); 
		$record{GRIDVIEW_PUBLISHER} = "GRIDVIEW_PUBLISHER_ERROR: parseLine No Match: $inline";
		return %record;
        }
	&logMessage(6, "parseLine: After breakup from Reqular Expression $1 $2 $3 $4 $5 $6 $7 $8 $9 $10 $11 $12 $13 $14 $15 $16");
        $record{DATE}   = $1;   $record{HOST}   = $2;
        $record{PROG}   = $3;   $record{EVNT}   = $4;
        $record{START}  = $5;   $record{USER}   = $6;
        $record{FILE}   = $7;   $record{BUFFER} = $8;
        $record{BLOCK}  = $9;   $record{NBYTES} = $10;
        $record{VOLUME} = $11;  $record{STREAMS}= $12;
        $record{STRIPES}= $13;  $record{DEST}   = $14;
        $record{TYPE}   = $15;  $record{CODE}   = $16;

        # Some buggy sites dont provide USER field
        # This is a check for it

        $record{USER} = 'Unknown' if( length $record{'USER'} == 0) ;

        # Getting FQDN for the HOST

        my $value = $record{HOST};
        my @tmp = gethostbyname($value);
        $value = $tmp[0] if(@tmp > 0 );
        $record{HOST} = $value;

        # Extracting PEER name

        $value = $record{DEST};
	$record{PEER} = $record{DEST};
        use Socket;
        @tmp = gethostbyaddr(Socket::inet_aton($value),AF_INET);
	if(@tmp > 0) {  $record{PEER} = $tmp[0];}

        # If operation Type is storage then Swap the HOST and DEST

        if ($record{TYPE} =~ /STOR/) {
                $record{SRC} = $record{PEER};
                $record{DEST} = $record{HOST};
        }
        else {
                $record{SRC} = $record{HOST};
                $record{DEST} = $record{PEER};
        }

        # Dont record 'ls' type of Activities.

	if($record{FILE} =~ /\/bin\/ls/) 
	{
		&logMessage (5, "parseLine: Ignore Input Line ignored $inline"); 
	        $record{GRIDVIEW_PUBLISHER} = "GRIDVIEW_PUBLISHER_IGNORE: parseLine File /bin/ls";
		return %record;
	}

        # Manipulating the DATE field to get the time
	my $end_time;
        if ( $record{DATE} =~ /(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)\.(\d*)/)
        {
                use POSIX;
                $record{ESECONDS} = POSIX::mktime($6 ,$5, $4, $3, $2 -1, $1 - 1900);
                $record{ESECONDS_MICRO} = $7;
                $end_time = $record{ESECONDS} + (0.000001 * $record{ESECONDS_MICRO});
                $record{END_DATE} = $1 . '-' . $2 . '-' . $3;
                $record{END_TIME} = $4 . ':' . $5 . ':' . $6;
        }

        # Manipulating the START field to get the time

        if ( $record{START} =~ /(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)\.(\d*)/)
        {
                use POSIX;
                $record{SSECONDS} = POSIX::mktime($6 ,$5, $4, $3, $2 -1, $1 - 1900);
                $record{SSECONDS_MICRO} = $7;
                $start_time = $record{SSECONDS} + (0.000001 * $record{SSECONDS_MICRO});
        }

        # Calculating the Transfer rate and upto precision of 4 decimal places

        $difftime = $end_time - $start_time;
        $difftime += 3600 if($difftime <= 0);
        $record{TXRATE} = sprintf("%0.4f", $record{NBYTES}/($difftime * 1048576));

#        print "\n";
#        print "DATE = $record{DATE} \nESECONDS = $record{ESECONDS}\n";
#        print "HOST = $record{HOST} \nSSCEONDS = $record{SSECONDS}\n";
#        print "PROG = $record{PROG} \nEVNT = $record{EVNT} \n";
#        print "START = $record{START} \nUSER = $record{USER} \n";
#        print "FILE = $record{FILE}   \nBUFFER = $record{BUFFER} \n";
#        print "BLOCK = $record{BLOCK} \nNBYTES = $record{NBYTES} \n";
#        print "VOLUME = $record{VOLUME} \nSTREAMS = $record{STREAMS} \n";
#        print "STRIPES = $record{STRIPES} \nDEST = $record{DEST} \n";
#        print "TYPE = $record{TYPE}  \nCODE = $record{CODE} \n";
#        print "PEER = $record{PEER} \nSRC = $record{SRC}\n";
#        print "END_DATE = $record{END_DATE} \nEND_TIME = $record{END_TIME}\n";
#        print "TXRATE = $record{TXRATE}\n";
	&logMessage(6, "parseLine: Succesfully Parsed the line $inline");
	$record{GRIDVIEW_PUBLISHER} = "GRIDVIEW_PUBLISHER_PUBLISH: parseLine Success";
	return %record;
}
